package com.itechnica.svg {
	import com.itechnica.svg.draw.CurveCommand;	
	import com.itechnica.svg.draw.LineCommand;	
	import com.itechnica.svg.draw.MoveCommand;	
	import com.itechnica.svg.draw.StyleCommand;	
	import com.itechnica.svg.draw.FillCommand;	
	
	import flash.geom.Point;
	import flash.xml.XMLNode;
	
	import com.itechnica.svg.draw.DrawingCommand;
	import com.itechnica.svg.geom.CubicBezierSVG;
	import com.itechnica.svg.geom.QuadraticBezierSVG;		

	public class PathToArray {

		private static const colors:Object = {blue:0x0000ff, green:0x008000, red:0xff0000, none:undefined, aliceblue:0xf0f8ff, antiquewhite:0xfaebd7, aqua:0x00ffff, aquamarine:0x7fffd4, azure:0xf0ffff, beige:0xf5f5dc, bisque:0xffe4c4, black:0x000000, blanchedalmond:0xffebcd, blueviolet:0x8a2be2, brown: 0xa52a2a, burlywood : 0xdeb887, cadetblue : 0x5f9ea0, chartreuse : 0x7fff00, chocolate : 0xd2691e, coral: 0xff7f50, cornflowerblue : 0x6495ed, cornsilk : 0xfff8dc, crimson : 0xdc143c, cyan : 0x00ffff, darkblue : 0x00008b, darkcyan : 0x008b8b, darkgoldenrod : 0xb8860b, darkgray : 0xa9a9a9, darkgreen : 0x006400, darkgrey : 0xa9a9a9, darkkhaki : 0xbdb76b, darkmagenta : 0x8b008b, darkolivegreen : 0x556b2f, darkorange : 0xff8c00, darkorchid : 0x9932cc, darkred : 0x8b0000, darksalmon : 0xe9967a, darkseagreen : 0x8fbc8f, darkslateblue : 0x483d8b, darkslategray : 0x2f4f4f, darkslategrey : 0x2f4f4f, darkturquoise : 0x00ced1, darkviolet : 0x9400d3, deeppink : 0xff1493, deepskyblue : 0x00bfff, dimgray : 0x696969, dimgrey : 0x696969, dodgerblue : 0x1e90ff, firebrick : 0xb22222, floralwhite : 0xfffaf0, forestgreen : 0x228b22, fuchsia : 0xff00ff, gainsboro : 0xdcdcdc, ghostwhite : 0xf8f8ff, gold : 0xffd700, goldenrod : 0xdaa520, gray : 0x808080, grey : 0x808080, greenyellow : 0xadff2f, honeydew : 0xf0fff0, hotpink : 0xff69b4, indianred : 0xcd5c5c, indigo : 0x4b0082, ivory : 0xfffff0, khaki : 0xf0e68c, lavender : 0xe6e6fa, lavenderblush : 0xfff0f5, lawngreen : 0x7cfc00, lemonchiffon : 0xfffacd, lightblue : 0xadd8e6, lightcoral : 0xf08080, lightcyan : 0xe0ffff, lightgoldenrodyellow : 0xfafad2, lightgray : 0xd3d3d3, lightgreen : 0x90ee90, lightgrey : 0xd3d3d3, lightpink : 0xffb6c1, lightsalmon : 0xffa07a, lightseagreen : 0x20b2aa, lightskyblue : 0x87cefa, lightslategray : 0x778899, lightslategrey : 0x778899, lightsteelblue : 0xb0c4de, lightyellow : 0xffffe0, lime : 0x00ff00, limegreen : 0x32cd32, linen : 0xfaf0e6, magenta : 0xff00ff, maroon : 0x800000, mediumaquamarine : 0x66cdaa, mediumblue : 0x0000cd, mediumorchid : 0xba55d3, mediumpurple : 0x9370db, mediumseagreen : 0x3cb371, mediumslateblue : 0x7b68ee, mediumspringgreen : 0x00fa9a, mediumturquoise : 0x48d1cc, mediumvioletred : 0xc71585, midnightblue : 0x191970, mintcream : 0xf5fffa, mistyrose : 0xffe4e1, moccasin : 0xffe4b5, navajowhite : 0xffdead, navy : 0x000080, oldlace : 0xfdf5e6, olive : 0x808000, olivedrab : 0x6b8e23, orange : 0xffa500, orangered : 0xff4500, orchid : 0xda70d6, palegoldenrod : 0xeee8aa, palegreen : 0x98fb98, paleturquoise : 0xafeeee, palevioletred : 0xdb7093, papayawhip : 0xffefd5, peachpuff : 0xffdab9, peru : 0xcd853f, pink : 0xffc0cb, plum : 0xdda0dd, powderblue : 0xb0e0e6, purple : 0x800080, rosybrown : 0xbc8f8f, royalblue : 0x4169e1, saddlebrown : 0x8b4513, salmon : 0xfa8072, sandybrown : 0xf4a460, seagreen : 0x2e8b57, seashell : 0xfff5ee, sienna : 0xa0522d, silver : 0xc0c0c0, skyblue : 0x87ceeb, slateblue : 0x6a5acd, slategray : 0x708090, slategrey : 0x708090, snow : 0xfffafa, springgreen : 0x00ff7f, steelblue : 0x4682b4, tan : 0xd2b48c, teal : 0x008080, thistle : 0xd8bfd8, tomato : 0xff6347, turquoise : 0x40e0d0, violet : 0xee82ee, wheat : 0xf5deb3, white : 0xffffff, whitesmoke : 0xf5f5f5, yellow : 0xffff00, yellowgreen : 0x9acd32};

		private var drawingCommands:Array;
		private var fill:Fill;
		private var stroke:Stroke;

		public function PathToArray(svgNode:XMLNode, dCmds:Array) {
			drawingCommands = dCmds;
			var svgCmds:Array = extractCmds(svgNode);
			makeDrawCmds(svgCmds);
		}

		/**
		 * @method extractCmds ()
		 * @param node (XMLNode) SVG path node
		 * @description Parse path node and convert to array of SVG drawing commands and data
		 *			eg, M,250.8,33.8,c,-33.6,-9.7,-42,19.1,-48.2,22.6,s,-27.9,2.2,-33.3,5.8,
		 *				c,-5.3,3.5,-17.3,23.5,-8.4,41.6
		 * @returns (Array) array of drawing commands
		 */	
		private function extractCmds(node:XMLNode):Array {
			var startColor:Number;
			var thisColor:Number;

			var hasFill:Boolean = false;
			var hasTransform:Boolean = false;
			var hasStroke:Boolean = false;
			var hasStrokeWidth:Boolean = false;
			var hasRotate:Boolean = false;
			var dstring:String = "";
			var rotation:Number;

			// is there a fill attribute, a transform attribute, a stroke attribute?
			for (var a:String in node.attributes) {
				if (a == "fill") {
					hasFill = true;
				} else if (a == "transform") {
					hasTransform = true;
				} else if (a == "stroke") {
					hasStroke = true;
				} else if (a == "stroke-width") {
					hasStrokeWidth = true;
				}
			}
			if (hasFill) {
				// parse for fill color specification
				// if a hex number is specified, startColor will be > 0
				// if a color name is specified, startColor will be 0
				startColor = getAttribute(node, "fill").indexOf("#") + 1;
				if (startColor == 0) {   
					// name specified instead of color number
					thisColor = colors[getAttribute(node, "fill")];
					// if (thisColor == undefined) {
					if (isNaN(thisColor)) {
						fill = new Fill(0, 0);  // set invisible if undefined
					} else {
						fill = new Fill(thisColor, 100);
					}
				} else {
					fill = new Fill(parseInt(getAttribute(node, "fill").substr(startColor, 6), 16), 100);
				}
			} else fill = new Fill(0xffffff, 100); 
		
			// stroke: color, width, alpha
			if (hasStroke) {
				// parse for stroke color specification
				startColor = getAttribute(node, "stroke").indexOf("#") + 1;		
				if (startColor == 0) {   
					// name specified instead of color number
					thisColor = colors[getAttribute(node, "stroke")];
					// if (thisColor == undefined) {
					if (isNaN(thisColor)) {
						stroke = new Stroke(0, 0, 0);
					} else {
						stroke = new Stroke(colors[getAttribute(node, "stroke")], 0, 100);
					}
				} else {
					stroke = new Stroke(parseInt(getAttribute(node, "stroke").substr(startColor, 6), 16), 0, 100);
				}
		  // if stroke is undefined, use invisible stroke
			} else stroke = new Stroke(0, 0, 0);
		
			if (hasStrokeWidth) stroke.width = Number(node.attributes["stroke-width"]);
	
			// if stroke and fill are both undefined, set fill to black 
			if (!hasFill && !hasStroke) {
				fill = new Fill(0, 100);
			}
		
			if (hasTransform) {
				// parse for rotation specification
				// hasRotate = getAttribute(node, "transform").indexOf("rotate");
				// if (hasRotate > -1) {
				hasRotate = getAttribute(node, "transform").indexOf("rotate") > -1;
				if (hasRotate) {
					var startRotate:Number = getAttribute(node, "transform").indexOf("(");
					var endRotate:Number = getAttribute(node, "transform").indexOf(")");
					rotation = parseInt(getAttribute(node, "transform").substr(startRotate + 1, endRotate - startRotate));
				} else {
					rotation = 0;
				}
			} else rotation = 0;
		
			// if commas included, is it Adobe Illustrator (no spaces) or SVG Factory/other?
			if (getAttribute(node, "d").indexOf(",") > -1) {  
				// has commas?
				if (getAttribute(node, "d").indexOf(" ") > -1) {  
					// yes, has spaces?
					// change spaces to commas, then deal as for Illustrator
					dstring = String2.replace(getAttribute(node, "d"), " ", ",");
				}  else {
					dstring = getAttribute(node, "d");
				}
			} else {  
				// no commas
				// get rid of extra spaces and change rest to commas
				dstring = getAttribute(node, "d");
				dstring = String2.shrinkSequencesOf(dstring, " ");
				dstring = String2.replace(getAttribute(node, "d"), " ", ",");
			}		
			dstring = String2.replace(dstring, "c", ",c,");
			dstring = String2.replace(dstring, "C", ",C,");
			dstring = String2.replace(dstring, "S", ",S,");
			dstring = String2.replace(dstring, "s", ",s,");
			// separate the z from the last element
			dstring = String2.replace(dstring, "z", ",z");
			// change the following if M can be mid-path
			dstring = String2.replace(dstring, "M", "M,");
			dstring = String2.replace(dstring, "L", ",L,");
			dstring = String2.replace(dstring, "l", ",l,");
			dstring = String2.replace(dstring, "H", ",H,");
			dstring = String2.replace(dstring, "h", ",h,");
			dstring = String2.replace(dstring, "V", ",V,");
			dstring = String2.replace(dstring, "v", ",v,");
			dstring = String2.replace(dstring, "Q", ",Q,");
			dstring = String2.replace(dstring, "q", ",q,");
			dstring = String2.replace(dstring, "T", ",T,");
			dstring = String2.replace(dstring, "t", ",t,");
			// Adobe includes no delimiter before negative numbers
			dstring = String2.replace(dstring, "-", ",-");
			// get rid of any dup commas we might have introduced
			dstring = String2.replace(dstring, ",,", ",");
			// get rid of spaces
			// (cr/lf's have to be removed before the xml object can be created,
			//  so that is done in xml.onData method)
			dstring = String2.replace(dstring, " ", "");
			dstring = String2.replace(dstring, "\t", "");

			return dstring.split(",");	
		}

		/**
		 * @method makeDrawCmds
		 * @param svgCmds (Array) array of svg draw commands (as output from extractCmds)
		 * @description Convert svg draw commands to array of ASVDrawing commands: drawingCommands
		 */	
		private function makeDrawCmds(svgCmds:Array):void {
			var j:Number = 0;
			var firstP:Point;
			var lastP:Point;
			var lastC:Point;
			var cmd:String;
			var сubicBezier:CubicBezierSVG;
		
			do {
				cmd = svgCmds[j++];
				switch (cmd) {
					case DrawingCommand.MOVE :
						// moveTo point
						firstP = lastP = new Point(Number(svgCmds[j]), Number(svgCmds[j + 1]));
						
						// drawingCommands.push([DrawingCommand.FILL, [fill.color, fill.alpha]]);
						drawingCommands.push(new FillCommand(fill.color, fill.alpha));
						
						// drawingCommands.push([DrawingCommand.STYLE, [stroke.width, stroke.color, stroke.alpha]]);
						drawingCommands.push(new StyleCommand(stroke.width, stroke.color, stroke.alpha));
						
						// drawingCommands.push([DrawingCommand.MOVE, [firstP.x, firstP.y]]);
						drawingCommands.push(new MoveCommand(firstP.clone()));
						
						j += 2;
						if (j < svgCmds.length && !isNaN(Number(svgCmds[j]))) {  
							do {
								// if multiple points listed, add the rest as lineTo points
								lastP = new Point(Number(svgCmds[j]), Number(svgCmds[j + 1]));
								
								// drawingCommands.push([DrawingCommand.LINE, [lastP.x, lastP.y]]);
								drawingCommands.push(new LineCommand(lastP.clone()));
								
								firstP = lastP;
								j += 2;
							} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						}
						break;
				
					case "l" :
						do {
							lastP = new Point(lastP.x + Number(svgCmds[j]), lastP.y + Number(svgCmds[j + 1]));
							
							// drawingCommands.push([DrawingCommand.LINE, [lastP.x, lastP.y]]);
							drawingCommands.push(new LineCommand(lastP.clone()));
							
							firstP = lastP;
							j += 2;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
				
					case DrawingCommand.LINE :
						do {
							lastP = new Point(Number(svgCmds[j]), Number(svgCmds[j + 1]));
							
							// drawingCommands.push([DrawingCommand.LINE, [lastP.x, lastP.y]]);					
							drawingCommands.push(new LineCommand(lastP.clone()));					
							
							firstP = lastP;
							j += 2;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
				
					case "h" :
						do {
							lastP = new Point(lastP.x + Number(svgCmds[j]), lastP.y);
							
							// drawingCommands.push([DrawingCommand.LINE, [lastP.x, lastP.y]]);
							drawingCommands.push(new LineCommand(lastP.clone()));
							
							firstP = lastP;
							j += 1;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
				
					case "H" :
						do {
							lastP = new Point(Number(svgCmds[j]), lastP.y);
							
							// drawingCommands.push([DrawingCommand.LINE, [lastP.x, lastP.y]]);
							drawingCommands.push(new LineCommand(lastP.clone()));
							
							firstP = lastP;
							j += 1;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
				
					case "v" :
						do {
							lastP = new Point(lastP.x, lastP.y + Number(svgCmds[j]));
							
							// drawingCommands.push([DrawingCommand.LINE, [lastP.x, lastP.y]]);
							drawingCommands.push(new LineCommand(lastP.clone()));
							
							firstP = lastP;
							j += 1;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
				
					case "V" :
						do {
							lastP = new Point(lastP.x, Number(svgCmds[j]));
							
							// drawingCommands.push([DrawingCommand.LINE, [lastP.x, lastP.y]]);
							drawingCommands.push(new LineCommand(lastP.clone()));
							
							firstP = lastP;
							j += 1;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
	
					case "q" :
						do {
							// control is relative to lastP, not lastC
							lastC = new Point(lastP.x + Number(svgCmds[j]), lastP.y + Number(svgCmds[j + 1]));
							lastP = new Point(lastP.x + Number(svgCmds[j + 2]), lastP.y + Number(svgCmds[j + 3]));
							
							// drawingCommands.push([DrawingCommand.CURVE, [lastC.x, lastC.y, lastP.x, lastP.y]]);
							drawingCommands.push(new CurveCommand(lastC.clone(), lastP.clone()));
							
							firstP = lastP;
							j += 4;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
				
					case "Q" :
						do {
							lastC = new Point(Number(svgCmds[j]), Number(svgCmds[j + 1]));					
							lastP = new Point(Number(svgCmds[j + 2]), Number(svgCmds[j + 3]));
							
							// drawingCommands.push([DrawingCommand.CURVE, [lastC.x, lastC.y, lastP.x, lastP.y]]);
							drawingCommands.push(new CurveCommand(lastC.clone(), lastP.clone()));
							
							firstP = lastP;
							j += 4;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
				
					case "c" :
						do {
							// don't save if c1.x=c1.y=c2.x=c2.y=0 
							if (!Number(svgCmds[j]) && !Number(svgCmds[j + 1]) && !Number(svgCmds[j + 2]) && !Number(svgCmds[j + 3])) {
					} else {
								сubicBezier = new CubicBezierSVG(new Point(lastP.x, lastP.y), new Point(lastP.x + Number(svgCmds[j]), lastP.y + Number(svgCmds[j + 1])), new Point(lastP.x + Number(svgCmds[j + 2]), lastP.y + Number(svgCmds[j + 3])), new Point(lastP.x + Number(svgCmds[j + 4]), lastP.y + Number(svgCmds[j + 5])));
								pushCubicBezierDrawCommands(сubicBezier);
								
								lastC = new Point(lastP.x + Number(svgCmds[j + 2]), lastP.y + Number(svgCmds[j + 3]));
								lastP = new Point(lastP.x + Number(svgCmds[j + 4]), lastP.y + Number(svgCmds[j + 5]));
								firstP = lastP;
							}
							j += 6;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
	
					case DrawingCommand.CURVE :
						do {
							// don't save if c1.x=c1.y=c2.x=c2.y=0 
							if (!Number(svgCmds[j]) && !Number(svgCmds[j + 1]) && !Number(svgCmds[j + 2]) && !Number(svgCmds[j + 3])) {
					} else {
								сubicBezier = new CubicBezierSVG(new Point(firstP.x, firstP.y), new Point(Number(svgCmds[j]), Number(svgCmds[j + 1])), new Point(Number(svgCmds[j + 2]), Number(svgCmds[j + 3])), new Point(Number(svgCmds[j + 4]), Number(svgCmds[j + 5])));
								pushCubicBezierDrawCommands(сubicBezier);
								
								lastC = new Point(Number(svgCmds[j + 2]), Number(svgCmds[j + 3]));
								lastP = new Point(Number(svgCmds[j + 4]), Number(svgCmds[j + 5]));
								firstP = lastP;
							}
							j += 6;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
				
					case "s" :
						do {
							// don't save if c1.x=c1.y=c2.x=c2.y=0 
							if (!Number(svgCmds[j]) && !Number(svgCmds[j + 1]) && !Number(svgCmds[j + 2]) && !Number(svgCmds[j + 3])) {
					} else {
								сubicBezier = new CubicBezierSVG(new Point(firstP.x, firstP.y), new Point(lastP.x + (lastP.x - lastC.x), lastP.y + (lastP.y - lastC.y)), new Point(lastP.x + Number(svgCmds[j]), lastP.y + Number(svgCmds[j + 1])), new Point(lastP.x + Number(svgCmds[j + 2]), lastP.y + Number(svgCmds[j + 3])));
								pushCubicBezierDrawCommands(сubicBezier);
								
								lastC = new Point(lastP.x + Number(svgCmds[j]), lastP.y + Number(svgCmds[j + 1]));
								lastP = new Point(lastP.x + Number(svgCmds[j + 2]), lastP.y + Number(svgCmds[j + 3]));
								firstP = lastP;
							}
							j += 4;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
				
					case DrawingCommand.STYLE :
						do {
							// don't save if c1.x=c1.y=c2.x=c2.y=0 
							if (!Number(svgCmds[j]) && !Number(svgCmds[j + 1]) && !Number(svgCmds[j + 2]) && !Number(svgCmds[j + 3])) {
					} else {
								сubicBezier = new CubicBezierSVG(new Point(firstP.x, firstP.y), new Point(lastP.x + (lastP.x - lastC.x), lastP.y + (lastP.y - lastC.y)), new Point(Number(svgCmds[j]), Number(svgCmds[j + 1])), new Point(Number(svgCmds[j + 2]), Number(svgCmds[j + 3])));
								pushCubicBezierDrawCommands(сubicBezier);

								lastC = new Point(Number(svgCmds[j]), Number(svgCmds[j + 1]));
								lastP = new Point(Number(svgCmds[j + 2]), Number(svgCmds[j + 3]));
								firstP = lastP;
							}
							j += 4;
						} while (j < svgCmds.length && !isNaN(Number(svgCmds[j])));
						break;
				
					case "z" :
					case "Z" :
						if (firstP.x != lastP.x || firstP.y != lastP.y) {
							
							// drawingCommands.push([DrawingCommand.LINE, [firstP.x, firstP.y]]);
							drawingCommands.push(new LineCommand(firstP.clone()));
						}
						j++;
						break;		
				} // end switch
			}  while (j < svgCmds.length);
		}
		
		private function pushCubicBezierDrawCommands(сubicBezier:CubicBezierSVG):void {
			const quadraticCurves:Array = [];
			сubicBezier.toQuadraticBezierArray(1, quadraticCurves);
			for (var i:int = 0;i < quadraticCurves.length; i++) {
				var curve:QuadraticBezierSVG = quadraticCurves[i] as QuadraticBezierSVG;
				if (curve!=null) {
					// drawingCommands.push([DrawingCommand.CURVE, [curve.control.x, curve.control.y, curve.end.x, curve.end.y]]);
					drawingCommands.push(new CurveCommand(curve.control.clone(), curve.end.clone()));
				} else {
					throw new Error("com.itechnica.svg.PathToArray.pushCubicBezierDrawCommands: incorrect curve type"); 
				}
			}
		}

		
		
		// UTILS

		private function getAttribute(node:XMLNode, attributeName:String):String {
			return node.attributes[attributeName];
		}
	}
}

internal class Fill {

	public var color:uint;
	public var alpha:Number;

	public function Fill(color:uint, alpha:Number) {
		initInstance(color, alpha);
	}

	private function initInstance(color:uint, alpha:Number):void {
		this.color = color;
		this.alpha = alpha;
	}
}

internal class Stroke {

	public var color:uint;
	public var alpha:Number;
	public var width:Number;

	public function Stroke(color:uint, alpha:Number, width:Number) {
		initInstance(color, alpha, width);
	}

	private function initInstance(color:uint, alpha:Number, width:Number):void {
		this.color = color;
		this.alpha = alpha;
		this.width = width;
	}
}











