package com.itechnica.svg.geom {
	import flash.geom.Point;		

	public class CubicBezierSVG {

		private var startPoint : Point;
		private var startControlPoint : Point;
		private var endControlPoint : Point;
		private var endPoint : Point;

		public function CubicBezierSVG(start:Point, startControl:Point, endControl:Point, end:Point) {
			initInstance(start, startControl, endControl, end);
		}
		
		private function initInstance(start : Point, startControl : Point, endControl : Point, end : Point) : void {
			startPoint = start;
			startControlPoint = startControl;
			endControlPoint = endControl;
			endPoint = end;
		}
		
		public function get start() : Point {
			return startPoint;
		}
		public function set start(value : Point):void {
			startPoint = value;
		}
		
		public function get startControl() : Point {
			return startControlPoint;
		}
		public function set startControl(value : Point):void {
			startControlPoint = value;
		}
		
		public function get endControl() : Point {
			return endControlPoint;
		}
		public function set endControl(value : Point):void {
			endControlPoint = value;
		}
		public function get end() : Point {
			return endPoint;
		}
		public function set end(value : Point):void {
			endPoint = value;
		}
		
		public function split():Array {
			const startMidpoint:Point = Point.interpolate(startPoint, startControlPoint, 0.5);
			const middleMidpoint:Point = Point.interpolate(startControlPoint, endControlPoint, 0.5);
			const endMidpoint:Point = Point.interpolate(endControlPoint, endPoint, 0.5);
			
			const startMiddleMidpoint:Point = Point.interpolate(startMidpoint, middleMidpoint, 0.5);
			const middleEndMidpoint:Point = Point.interpolate(middleMidpoint, endMidpoint, 0.5);
			const centerMidpoint:Point = Point.interpolate(startMiddleMidpoint, middleEndMidpoint, 0.5);
		
			return [
				new CubicBezierSVG(startPoint.clone(), startMidpoint, startMiddleMidpoint, centerMidpoint), 
				new CubicBezierSVG(centerMidpoint, middleEndMidpoint, endMidpoint, endPoint.clone())
				];
		}
		
		/** 
		* @method getQuadBez_RP (Penner)
		* @description  Approximates a cubic bezier with as many quadratic bezier segments (n) as required 
		*			to achieve a specified tolerance
		* @param p1 (Object) endpoint
		* @param c1 (Object) 1st control point
		* @param c2 (Object) 2nd control point
		* @param p2 (Object) endpoint
		* @param k: tolerance (low number = most accurate result)
		* @param qcurves (Array) will contain array of quadratic bezier curves, each element containing
		*		p1x, p1y, cx, cy, p2x, p2y
		*/
		public function toQuadraticBezierArray(k:Number, quadraticCurves:Array):void {
			const source:CubicBezierSVG = this;
			// find intersection between bezier arms
			var startArm : LineSVG = new LineSVG(source.start, source.startControl);
			var endArm : LineSVG = new LineSVG(source.endControl, source.end);
			var s:Point = startArm.getLineIntersection(endArm);
			// find distance between the midpoints
			var dx:Number = (source.start.x + source.end.x + s.x * 4 
							- (source.startControl.x + source.endControl.x) * 3) * .125;
			var dy:Number = (source.start.y + source.end.y + s.y * 4 
							- (source.startControl.y + source.endControl.y) * 3) * .125;
			// split curve if the quadratic isn't close enough
			if (dx*dx + dy*dy > k) {
				var halves:Array = source.split();
				var firstHalf:CubicBezierSVG = halves[0] as CubicBezierSVG; 
				var secondHalf:CubicBezierSVG = halves[1] as CubicBezierSVG;
				// recursive call to subdivide curve
				firstHalf.toQuadraticBezierArray (k, quadraticCurves);
				secondHalf.toQuadraticBezierArray(k, quadraticCurves);
			} else {
				// end recursion by saving points
				var quadraticStart:Point = new Point(source.start.x, source.start.y);
				var quadraticControl:Point = new Point(s.x, s.y);
				var quadraticEnd:Point = new Point(source.end.x, source.end.y);
				var quadraticBezier:QuadraticBezierSVG = 
					new QuadraticBezierSVG(quadraticStart, quadraticControl, quadraticEnd);
				quadraticCurves.push(quadraticBezier);
			}
		}
		
		
		
	}
}


















